@@ -98,6 +98,7 @@ Dataset.template <- function(id, env = solvebio:::.solveEnv) {
 #' Returns one page of documents from a SolveBio dataset and processes the response.
 #' @param id The ID of a SolveBio dataset, or a Dataset object.
 #' @param filters (optional) Query filters.
+#' @param row.names (optional) Force data frame row name ordering.
 #' @param env (optional) Custom client environment.
 #' @param ... (optional) Additional query parameters (e.g. limit, offset).
 #'
@@ -109,7 +110,7 @@ Dataset.template <- function(id, env = solvebio:::.solveEnv) {
 #' \url{https://docs.solvebio.com/}
 #'
 #' @export
-Dataset.data <- function(id, filters, env = solvebio:::.solveEnv, ...) {
+Dataset.data <- function(id, filters, row.names = NULL, env = solvebio:::.solveEnv, ...) {
     if (missing(id) || !(class(id) %in% c("Dataset", "numeric", "integer", "character"))) {
         stop("A dataset ID (or object) is required.")
     }
@@ -136,7 +137,7 @@ Dataset.data <- function(id, filters, env = solvebio:::.solveEnv, ...) {
 
     tryCatch({
         res <- .request('POST', path=path, body=body, env=env)
-        return(formatSolveBioQueryResponse(res))
+        return(formatSolveBioQueryResponse(res, row.names=row.names))
     }, error = function(e) {
         cat(sprintf("Query failed: %s\n", e$message))
     })
@@ -166,6 +167,9 @@ Dataset.query <- function(id, paginate=FALSE, env = solvebio:::.solveEnv, ...) {
     params$id <- id
     params$env <- env
 
+    # Retrieve the list of ordered fields
+    params$row.names <- do.call(Dataset.fields, list(id, limit=1000))$data$name
+
     # Retrieve the first page of results
     response <- do.call(Dataset.data, params)
     df <- response$result
@@ -217,6 +221,7 @@ Dataset.fields <- function(id, env = solvebio:::.solveEnv, ...) {
     if (missing(id)) {
         stop("A dataset ID is required.")
     }
+
     if (class(id) == "Dataset" || class(id) == "Object") {
         id <- id$id
     }
